﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Win.UltraWinTree;

namespace WindowsFormsApp2
{
    public partial class Form1 : Form
    {
        private dstDemo _dstDemo;
        private UltraTreeColumnSet colsetDatePart;
        private UltraTreeColumnSet colsetRequirement;
        
        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            // clearing the tree
            this.ultraTree1.Nodes.Clear();
            
            //Setting up
            this.ultraTree1.ViewStyle = ViewStyle.FreeForm;
            
            var columnsetDatePart = new UltraTreeColumnSet();
            columnsetDatePart.Columns.Add("Name");
            columnsetDatePart.Columns.Add("A");
            columnsetDatePart.Columns.Add("B");
            columnsetDatePart.Columns.Add("C");

            var columnSetRequirement = new UltraTreeColumnSet();
            columnSetRequirement.Columns.Add("Name");
            columnSetRequirement.Columns.Add("1");
            columnSetRequirement.Columns.Add("2");

            //Drawing
            var ndeYear = this.ultraTree1.Nodes.Add();
            ndeYear.Override.ColumnSet = columnSetRequirement;
            ndeYear.Cells["Name"].Value = "Year";
            ndeYear.Cells["1"].Value = "11";
            ndeYear.Cells["2"].Value = "21";
            ndeYear.Nodes.Override.ColumnSet = columnsetDatePart;

            var ndeHalfYear = ndeYear.Nodes.Add();
            ndeHalfYear.Override.ColumnSet = columnSetRequirement;
            ndeHalfYear.Cells["Name"].Value = "Half Year";
            ndeHalfYear.Cells["1"].Value = "11";
            ndeHalfYear.Cells["2"].Value = "21";
            ndeHalfYear.Cells["Name"].Column.AutoSizeMode = ColumnAutoSizeMode.AllNodes;

            var childNode = ndeYear.Nodes.Add();
            childNode.Cells["Name"].Value = "YearRequirement1";
            childNode.Cells["A"].Value = "A1";
            childNode.Cells["B"].Value = "B1";
            childNode.Cells["C"].Value = "C1";
            childNode.Cells["Name"].Column.AutoSizeMode = ColumnAutoSizeMode.AllNodes;

            childNode = ndeYear.Nodes.Add();
            childNode.Cells["Name"].Value = "YearRequirement2";
            childNode.Cells["A"].Value = "A2";
            childNode.Cells["B"].Value = "B2";
            childNode.Cells["C"].Value = "C2";
            childNode.Cells["Name"].Column.AutoSizeMode = ColumnAutoSizeMode.AllNodes;

            //Add Halfyear requirements
            childNode = ndeHalfYear.Nodes.Add();
            childNode.Override.ColumnSet = columnsetDatePart;
            childNode.Cells["Name"].Value = "HalfyearRequirement1";
            childNode.Cells["A"].Value = "A3";
            childNode.Cells["B"].Value = "B3";
            childNode.Cells["C"].Value = "C3";
            childNode.Cells["Name"].Column.AutoSizeMode = ColumnAutoSizeMode.AllNodes;

            this.ultraTree1.ExpandAll();

        }

        private void button2_Click(object sender, EventArgs e)
        {
            // clearing the tree
            this.ultraTree1.Nodes.Clear();

            SetupDataTables();
            SetupColumnsets();
            DrawTree();
        }


        /// <summary>
        /// populates the 2 datatables oft the dataset with sample data
        /// </summary>
        private void SetupDataTables()
        {
            _dstDemo = new dstDemo();

            var rowYear = _dstDemo.DateParts.AddDatePartsRow("Year", (string) null);
            var rowHalfyear =_dstDemo.DateParts.AddDatePartsRow("Halfyear", "Year");
            var rowQuarter =_dstDemo.DateParts.AddDatePartsRow("Quarter", "Halfyear");
            var rowMonth =_dstDemo.DateParts.AddDatePartsRow("Month", "Quarter");

            _dstDemo.Requirements.AddRequirementsRow("YearRequirement1", rowYear);
            _dstDemo.Requirements.AddRequirementsRow("YearRequirement2", rowYear);
            _dstDemo.Requirements.AddRequirementsRow("YearRequirement3", rowYear);
            _dstDemo.Requirements.AddRequirementsRow("HalfyearRequirement1", rowHalfyear);
            _dstDemo.Requirements.AddRequirementsRow("HalfyearRequirement2", rowHalfyear);
            _dstDemo.Requirements.AddRequirementsRow("HalfyearRequirement3", rowHalfyear);
            _dstDemo.Requirements.AddRequirementsRow("HalfyearRequirement4", rowHalfyear);
            _dstDemo.Requirements.AddRequirementsRow("QuarterRequirement1", rowQuarter);
            _dstDemo.Requirements.AddRequirementsRow("QuarterRequirement2", rowQuarter);
            _dstDemo.Requirements.AddRequirementsRow("MonthRequirement1", rowMonth);
            _dstDemo.Requirements.AddRequirementsRow("MonthRequirement2", rowMonth);
            _dstDemo.Requirements.AddRequirementsRow("MonthRequirement3", rowMonth);
        }


        private void SetupColumnsets()
        {
            colsetDatePart = new UltraTreeColumnSet();
            colsetDatePart.Columns.Add("Name");
            colsetDatePart.Columns.Add("ParentDatePart");

            colsetRequirement = new UltraTreeColumnSet();
            colsetRequirement.Columns.Add("Name");
            colsetRequirement.Columns.Add("DatePart");
        }


        /// <summary>
        /// draws the tree based on the datatables in the dataset
        /// </summary>
        private void DrawTree()
        {
            //Draw Hierarchies
            this.ultraTree1.ViewStyle = ViewStyle.FreeForm;
            foreach (var rowDatePart in _dstDemo.DateParts)
            {
                //Build node
                var nde = new UltraTreeNode(key:rowDatePart.Name) {Override = {ColumnSet = colsetDatePart}};
                foreach (var column in colsetDatePart.Columns)
                {
                    nde.SetCellValue(column, rowDatePart[column.Key]); //colset columns and datarow columns match 1:1!
                }

                //Insert node hierarchically
                if (rowDatePart.IsParentDatePartNull())
                {
                    //Rootnode
                    ultraTree1.Nodes.Add(nde);
                }
                else
                {
                    //Childnode
                    var ndeParent = ultraTree1.GetNodeByKey(rowDatePart.ParentDatePart);
                    ndeParent.Nodes.Add(nde);
                }
                // add colsetRequirement set for each of their children
                // since the each colsetDatePart node would have its set overriden to colsetDatePart this would not apply to them
                nde.Nodes.Override.ColumnSet = colsetRequirement;
                
            }

            //Draw Requirements
            foreach (var rowRequirement in _dstDemo.Requirements)
            {
                //Build node
                var nde = new UltraTreeNode(key: rowRequirement.Name); // {Override = {ColumnSet  = colsetRequirement}};
                // get node parent
                var ndeParent = ultraTree1.GetNodeByKey(rowRequirement.DatePart);                               

                //Insert node hierarchically. Requirements are always children of a Datepart node.
                ndeParent.Nodes.Add(nde);

                // add the values to each cell moved after the node has been already created and addded to the tree
                foreach (var column in colsetRequirement.Columns)
                {
                    nde.SetCellValue(column, rowRequirement[column.Key]); //colset columns and datarow columns match 1:1!
                }
            }

            ultraTree1.ExpandAll(ExpandAllType.Always);
        }
    }
}
